import logging
from fastapi import FastAPI
from app.routes import meta, forecasts, aggregations

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s -%(levelname)s - %(message)s",
    handlers=[
        logging.StreamHandler()
    ]
)

app = FastAPI(
    title="AtmoSwing Web Forecast API",
    description="API to provide forecasts generated by AtmoSwing.",
    version="1.0.0",
)

# Include the routes
app.include_router(meta.router, prefix="/meta", tags=["Metadata"])
app.include_router(forecasts.router, prefix="/forecasts", tags=["Data from a single forecast"])
app.include_router(aggregations.router, prefix="/aggregations", tags=["Aggregated forecast data"])
